/* PRO-CITE_EXPORT.rexx  version 1.1 */
/* This macro can be run from the CLI, SHELL or WorkBench and */
/* acts on a text or word processor-based citation database from */
/* which you wish to export citations in PRO-CITE format.  The macro */
/* generates a text file that can be imported into PRO-CITE on  */
/* MacIntosh or MS-DOS platforms or into ENDNOTE on the MacIntosh. */

/* open the citation text file (database) for import */
/* NOTE: change the disk/drive designations below to suit your setup */
in_text = getfile(,,'ram:',,'Select citation import file');
if (in_text = '') then exit;
if (open(importfile,in_text,'R') ~= 1) then;
  do;
    say 'I could not open the file ' || in_text || ' for importing.';
    exit;
  end;

/* open the text file for export */
out_text = getfile(,,'ram:',,'Select PRO-CITE export name');
if (out_text = '') then exit;
if (open(exportfile,out_text,'W') ~= 1) then;
  do;
    say 'I could not open the file ' || out_text || ' for exporting.';
    exit;
  end;


/* get the first record */
Tab = d2c(9);
do j = 1 to 5;
  rec.j.value = readln(importfile);
end;


/* Start output loop for citations */
do while (~EOF(importfile));


/* Extract author names & reconstruct author string */
  authors = rec.1.value;
  author_string = '';
  n = 1;
  if (length(authors) >= 2) then;
    do;
      do forever;
        parse var authors surname.n ',' authors;
        surname.n = strip(surname.n,'B');
        if surname.n == '' then leave;
        parse var authors initials.n ',' authors;
        initials.n = compress(initials.n);
        new_initials = '';
        do k = 1 to length(initials.n);
           new_initials = (new_initials || (substr(initials.n,k,1) || ' '));
           fix_pos = index(new_initials,'J r');
           if fix_pos > 0 then;
             new_initials = delstr(new_initials,(fix_pos + 1),1);
        end;
        new_initials = translate(new_initials, '.', ' ');
        author_string = (author_string || surname.n || ', ' || new_initials || "//");
      end;
      author_string = left(author_string,(length(author_string)-2));
    end;
  else author_string = '';


/* Extract year and title */
   year = rec.2.value;
  title = rec.3.value;


/* Extract volume, first and last page numbers */
  parse var rec.4.value junk ',' volume ',' first ',' last ',';
  volume = compress(volume);
   first = compress(first);
    last = compress(last);
   pages = (first || '-' || last);

/* Extract full journal name */
  journal = strip(rec.5.value,'L','*');


/* Export the citation in the PRO-CITE format */
  citation = ('@D' || Tab || author_string || Tab || title || Tab || journal || Tab || year);
  citation = (citation || Tab || volume || Tab || Tab || pages || Tab || Tab || Tab || Tab);
  foo = writeln(exportfile,citation);
  say citation;

/* get the next record */
  do loop = 1 to 5;
    rec.loop.value = readln(importfile);
  end;
end;

/* exit macro */
say 'Exported PRO-CITE style citations in file ' || out_text;
exit;
